% Written by Amit Hochman.
% Released under the Gnu Public License.
% Last updated on: 6/25/2009.
function [integral, oMasks, oSdpTh, oSdpDn, oBcD, oBcU, oSc, oIm, oPole] = calc_si_integral(vX, vY, x_s, y_s, k0, Er_d, Er_u, points, sComp, isMSource, qsRule)
nX = length(vX);
integral = zeros(nX, 1);
oMasks = [];
bNonZero = (~isMSource & (strcmpi(sComp, 'Ez') | strcmpi(sComp, 'Hx') | strcmpi(sComp, 'Hy'))) |...
    (isMSource & (strcmpi(sComp, 'Hz') | strcmpi(sComp, 'Ex') | strcmpi(sComp, 'Ey')));
if bNonZero
    [oSdpTh, oSdpDn, oBcD, oBcU, oSc, oIm, oPole, oMasks] = calc_contours(vX, vY, x_s, y_s, k0, Er_d, Er_u, points, sComp, isMSource, qsRule);
    i_sdp_th = integral;
    i_sdp_dn = integral;
    i_bc_u = integral;
    i_bc_d = integral;
    i_sc = integral;
    if ~isempty(oSdpTh)
        E_sdp = integrand_vect(oSdpTh, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm);
        i_sdp_th(oSdpTh.bv) = sum(E_sdp.*oSdpTh.jac,2);
    end
    if ~isempty(oSc)
        E_sc = integrand_vect(oSc, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm);
        i_sc(oSc.bv) = sum(E_sc.*oSc.jac,2);
    end
    if ~isempty(oSdpDn)
        E_sdp = integrand_vect(oSdpDn, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm);
        i_sdp_dn(oSdpDn.bv) = sum(E_sdp.*oSdpDn.jac,2);
    end
    if ~isempty(oBcD)
        bv = oBcD{1}.bv;
        nKx = size(oBcD{1}.kx,2);
        mX = repmat(vX(bv), 1, nKx);
        phase_x = exp(-j*abs(mX-x_s).*oBcD{1}.kx);
        E_bcR = integrand_vect(oBcD{2}, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm)./phase_x;
        E_bcL = integrand_vect(oBcD{1}, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm)./phase_x;
        E_bcD = (E_bcR-E_bcL).*phase_x;

        % handle leaky pole in Mag. source
        bDown = 1;
        bvActive = oPole.vActive_d & bv;
        if any(bvActive)
            bvActiveInBv = bvActive(bv);
            vRes = calc_residue_vect(vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, oIm, oPole, bDown, bvActive);
            vGamma = oPole.vGamma_d(bvActive);
            vStep = oPole.vStep_d(bvActive);
            vSing = oPole.vSing_d(bvActive);
            mGamma = repmat(vGamma, 1, nKx);
            mStep = repmat(vStep, 1, nKx);
            mRes = repmat(vRes, 1, nKx);
            mSing = repmat(vSing, 1, nKx);
            k_plsmin = k0*sqrt(Er_d);
            k_pole = oPole.k_pole;
            mT = real((oBcD{1}.kx(bvActiveInBv,:)/k_plsmin-1)./mStep);
            mT_pole = (k_pole./k_plsmin-1)./mStep;
            E_pole = (mRes./(mT-mT_pole).*(mT).^mSing).*exp(-mGamma.*mT);
            E_bcD_regular = E_bcD(bvActiveInBv,:) - E_pole;
            dKxdT = k_plsmin*vStep;
            pole_ana = dKxdT.*vRes.*calc_pole_ana(vGamma, mT_pole(:,1), vSing);
            i_bc_d(bvActive) = sum(E_bcD_regular.*oBcD{1}.jac(bvActiveInBv,:),2) + pole_ana;
        end
        if any(~bvActive)
            bvNotActiveInBv = ~bvActive(bv);
            i_bc_d(bv & ~bvActive) = sum(E_bcD(bvNotActiveInBv,:).*oBcD{1}.jac(bvNotActiveInBv,:),2);
        end
    end
    if ~isempty(oBcU)
        bv = oBcU{1}.bv;
        nKx = size(oBcU{1}.kx,2);
        mX = repmat(vX(bv), 1, nKx);
        phase_x = exp(-j*abs(mX-x_s).*oBcU{1}.kx);
        E_bcR = integrand_vect(oBcU{2}, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm)./phase_x;
        E_bcL = integrand_vect(oBcU{1}, vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, isMSource, oIm)./phase_x;
        E_bcU = (E_bcR-E_bcL).*phase_x;
        % handle leaky pole in Mag. source
        bDown = 0;
        bvActive = oPole.vActive_u & bv;
        if any(bvActive)
            bvActiveInBv = bvActive(bv);
            vRes = calc_residue_vect(vX, vY, x_s, y_s, k0, Er_d, Er_u, sComp, oIm, oPole, bDown, bvActive);
            vGamma = oPole.vGamma_u(bvActive);
            vStep = oPole.vStep_u(bvActive);
            vSing = oPole.vSing_u(bvActive);
            mGamma = repmat(vGamma, 1, nKx);
            mStep = repmat(vStep, 1, nKx);
            mRes = repmat(vRes, 1, nKx);
            mSing = repmat(vSing, 1, nKx);

            k_plsmin = k0*sqrt(Er_u);
            k_pole = oPole.k_pole;
            mT = real((oBcU{1}.kx(bvActiveInBv,:)/k_plsmin-1)./mStep);
            mT_pole = (k_pole./k_plsmin-1)./mStep;
            E_pole = (mRes./(mT-mT_pole).*(mT).^mSing).*exp(-mGamma.*mT);
            E_bcU_regular = E_bcU(bvActiveInBv,:) - E_pole;
            dKxdT = k_plsmin*vStep;
            pole_ana = dKxdT.*vRes.*calc_pole_ana(vGamma, mT_pole(:,1), vSing);
            i_bc_u(bvActive) = sum(E_bcU_regular.*oBcU{1}.jac(bvActiveInBv,:),2) + pole_ana;
        end
        if any(~bvActive)
            bvNotActiveInBv = ~bvActive(bv);
            i_bc_u(bv & ~bvActive) = sum(E_bcU(bvNotActiveInBv,:).*oBcU{1}.jac(bvNotActiveInBv,:),2);
        end
    end
    im = zeros(size(integral));
    bvIm = oIm.Il ~= 0;
    if any(bvIm)
        im(bvIm) = oIm.Il(bvIm).*LS_space(vX(bvIm), vY(bvIm), oIm.x_s(bvIm), oIm.y_s(bvIm), k0, oIm.Er(bvIm), sComp, isMSource);
    end
    integral = i_sdp_th + i_sdp_dn + i_bc_u + i_bc_d + im + i_sc;
end
