% [field, nPoints] = ndsdp(EvalAt, Source, permittivityLowerRegion, permittivityUpperRegion, componentString, Options)
% Calculate a Cartesian field component due to a line-source in the
% presence of a dielectric half-space.
%
% Inputs:
% EvalAt.x, EvalAt.y - vectors or matrices of the coordinates at which the
%                      fields are evaluated.
% Source.x, Source.y - source coordinates.
% Source.polarizationString - 'TM' for an electric line-source, 'TE' for a
%                             magnetic one.
% permittivityLowerRegion, - relative permittivities of the y<0 and y>0 regions.
% permittivityUpperRegion
% componentString - string specifying the component to be calculated.
%                   Allowed values are: 'Ex', 'Ey', 'Ez', 'Hx', 'Hy',
%                   'Hz', 'Dx', 'Dy', 'Dz'.
% Options.quasistaticRule - string specifying the quadrature rule used for 
%                           the quasi-static case. Allowed values are: 
%                           'discretization' and 'Legendre' (default).
%                           'discretization' usually yields better accuracy
%                           but the generation of the rule may take longer.
% Options.useAdaptive - if true, change the number of points adaptively. If
%                       false, use Options.nPoints for all observation points.
% If options.useAdaptive == true, the following options must be set:
% Options.tol - Maximum relative error for adaptive scheme.
% Options.maxPoints - Max. number of integration points in the adaptive scheme.
% If options.useAdaptive == false, the following option must be set:
% Options.nPoints - number of integration points (should be even). 
%
% Outputs: 
% field - field component specified by componentString.
% nPoints - number of integration points used. 
% If Options.useAdaptive == true, the number of integration points used
% for each observation point is return in nPoints, which is the same size
% as EvalAt.x. If Options.useAdaptive == false, nPoints = Options.nPoints.
% ------------------------------------------------------------------------
% Written by Amit Hochman.
% Released under the Gnu Public License.
% Last updated on: 6/25/2009.
function [field, nPoints] = ndsdp(EvalAt, Source, permittivityLowerRegion, ...
    permittivityUpperRegion, componentString, Options)
% Set options
if ~isfield(Options, 'quasistaticRule')
    Options.quasistaticRule = 'Legendre';
end
if Options.useAdaptive
    [field, nPoints] = adaptive_ndsdp(EvalAt, Source, permittivityLowerRegion,...
        permittivityUpperRegion, componentString, Options);
    return
else
    nPoints = Options.nPoints;
end
k0 = 2*pi/Source.lambda;
isMagneticSource = ~(lower(Source.polarizationString(2)) == 'm');
% Handle equal permittivities case:
if permittivityLowerRegion == permittivityUpperRegion
    field = LS_space(EvalAt.x, EvalAt.y, Source.x, Source.y, k0, permittivityLowerRegion,...
        componentString, isMagneticSource);
    nPoints = 1;
    return
end

if lower(componentString(1)) == 'd'
    componentString(1) = 'E';
    bD = 1;
else
    bD = 0;
end
x = EvalAt.x(:);
y = EvalAt.y(:);
xSrc = Source.x;
ySrc = Source.y;
if abs(ySrc/Source.lambda) < 1e-5; % source right on the boundary is not handled well.
    ySrc = 1e-5;
end
EvalAtSwitched = EvalAt; % if ySrc>0 permittivites are switched, because code only handles ySrc<0
if ySrc > 0
    permittivityLowerRegionSwitched = permittivityUpperRegion;
    permittivityUpperRegionSwitched = permittivityLowerRegion;
    ySrcSwitched = -ySrc;
    xSwitched = x + 2*(xSrc - x);
    ySwitched = -y;
    if (componentString(2) == 'x') || (componentString(2) == 'y')
        sgn = -1;
    else
        sgn = 1;
    end
    EvalAtSwitched.y = -EvalAtSwitched.y;
    EvalAtSwitched.x = EvalAt.x + 2*(xSrc - EvalAt.x);
else
    permittivityLowerRegionSwitched = permittivityLowerRegion;
    permittivityUpperRegionSwitched = permittivityUpperRegion;
    ySrcSwitched = ySrc;
    xSwitched = x;
    ySwitched = y;
    sgn = 1;
end
integral = calc_si_integral(xSwitched, ySwitched, xSrc, ySrcSwitched, k0,...
    permittivityLowerRegionSwitched, permittivityUpperRegionSwitched, Options.nPoints,...
    componentString, isMagneticSource, Options.quasistaticRule);
addIncident = (sign(EvalAtSwitched.y) == sign(ySrcSwitched)) |...
    (EvalAtSwitched.y==0); % Incident, or direct, field needed for y<0.
if any(addIncident)
    incidentField = LS_space(EvalAtSwitched.x, EvalAtSwitched.y, xSrc, ...
        ySrcSwitched, k0, permittivityLowerRegionSwitched, componentString, isMagneticSource);
    incidentField = incidentField.*addIncident;
else
    incidentField = 0;
end
field = sgn*(integral + incidentField(:));
[rows, cols] = size(EvalAt.x);
field = reshape(field, rows, cols);
if bD
    bUp = EvalAt.y > 0;
    field(bUp) = field(bUp)*permittivityUpperRegion;
    field(~bUp) = field(~bUp)*permittivityLowerRegion;
end
